#include "ast.h"

static char*
strip_quotes(char *str)
{
  // if we have a char * that is
  // "hello"
  // we will return it without surrounding quotes
  // hello
  size_t len;
  char *str2;
  len = strnlen(str, 1024);

  str2 = strdup(str);
  str2++;
  str2[len-2] = '\0';

  return str2;
}

struct value_type*
new_value_array(struct List *elements)
{
  struct value_type *v = malloc(sizeof(struct value_type));

  v->type = ARRAY_TYPE;
  v->data = elements;

  return v;
}

struct value_type*
new_value_object(struct List *members)
{
  struct value_type *v = malloc(sizeof(struct value_type));

  v->type = OBJECT_TYPE;
  v->data = members;

  return v;
}

struct value_type*
new_value_string(char *string)
{
  struct value_type *v = malloc(sizeof(struct value_type));

  v->type = STRING_TYPE;
  v->data = strip_quotes(string);

  return v;
}

struct value_type*
new_value_integer(int i)
{
  struct value_type *v = malloc(sizeof(struct value_type));
  int *ii = malloc(sizeof(int));

  *ii = i;

  v->type = INTEGER_TYPE;
  v->data = ii;

  return v;
}

struct value_type*
new_value_boolean(int b)
{
  struct value_type *v = malloc(sizeof(struct value_type));
  int *bb = malloc(sizeof(int));

  *bb = b;

  v->type = BOOLEAN_TYPE;
  v->data = bb;

  return v;
}

struct value_type*
new_value_number(float f)
{
  struct value_type *v = malloc(sizeof(struct value_type));
  float *ff = malloc(sizeof(float));

  *ff = f;

  v->type = NUMBER_TYPE;
  v->data = ff;

  return v;
}

struct value_type*
new_value_null()
{
  struct value_type *v = malloc(sizeof(struct value_type));
  v->type = NULL_TYPE;
  v->data = NULL;

  return v;
}

struct List *
new_members()
{
 struct List *members = malloc(sizeof(struct List));
 list_init(members, member_match, member_free);
 return members;
}

struct member_type*
new_member_type(char *name, struct value_type *value)
{
  struct member_type *m = malloc(sizeof(struct member_type));
  size_t len;

  m->name = strip_quotes(name);
  m->value = value;

  return m;
}

void member_free(void *data)
{
  free(data);
}

int member_match(void *data1, void *data2)
{
  return data1 == data2;
}

void element_free(void *data)
{
  free(data);
}

int element_match(void *data1, void *data2)
{
  return data1 == data2;
}

struct List *
new_elements()
{
  struct List *elements = malloc(sizeof(struct List));
  list_init(elements, member_match, member_free);
  return elements;
}

int
new_array_element(struct List *elements, struct value_type *element)
{
  return list_add_last(elements, element);
}

int
new_object_member(struct List *members, struct member_type *member)
{
  return list_add_last(members, member);
}

void
json_print_recursive(struct value_type *value)
{
  struct List* members;
  struct List* elements;
  struct member_type *member;

  char *str;
  int *integer;
  float* number;

  if(value == NULL)
  {
    perror("value is null!?\n");
    return;
  }

  switch(value->type)
  {
  case OBJECT_TYPE:
    members = value->data;

    if(members == NULL)
    {
      printf("{}");
      break;
    }

    printf("{");
    for(int i=0; i<list_size(members); i++)
    {
      member = list_get_index(members, i);
      printf("\"%s\":", member->name);
      json_print_recursive(member->value);
      if(i < (list_size(members)-1))
      {
        printf(",");
      }
    }
    printf("}");

    break;
  case ARRAY_TYPE:
    elements = value->data;

    printf("[");
    for(int i=0; i<list_size(elements); i++)
    {
      value = list_get_index(elements, i);
      json_print_recursive(value);
      if(i < (list_size(elements)-1))
      {
        printf(",");
      }
    }
    printf("]");

    break;
  case STRING_TYPE:
    str = value->data;
    printf("\"%s\"", str);
    break;
  case INTEGER_TYPE:
    integer = value->data;
    printf("%d", *integer);
    break;
  case NUMBER_TYPE:
    number = value->data;
    printf("%f", *number);
    break;
  case BOOLEAN_TYPE:
    integer = value->data;
    if(*integer != 0)
      printf("true");
    else
      printf("false");
    break;
  case NULL_TYPE:
      printf("null");
    break;
  default:
    perror("unknown value type");
  }
  return;
}
