#ifndef AST_H
#define AST_H

#include <stdlib.h>
#include <stdio.h>
#include "list.h"

enum VALUE_TYPE
{
  OBJECT_TYPE,
  ARRAY_TYPE,
  STRING_TYPE,
  NUMBER_TYPE,
  INTEGER_TYPE,
  BOOLEAN_TYPE,
  NULL_TYPE
};

struct value_type
{
  enum VALUE_TYPE type;
  // the data pointer can be a primitive like a string, int, boolean or null
  // or to an array or object
  void *data;
};

struct member_type
{
  char *name;
  struct value_type *value;
};

void
member_free(void *data);

int
member_match(void *data1, void *data2);

struct value_type*
new_value_array(struct List *elements);

struct value_type*
new_value_object(struct List *members);

struct value_type*
new_value_string(char *string);

struct value_type*
new_value_integer(int i);

struct value_type*
new_value_boolean(int i);

struct value_type*
new_value_number(float f);

struct value_type*
new_value_null();

struct List *
new_members();

struct member_type*
new_member_type(char *name, struct value_type *value);

struct List *
new_elements();

int
new_array_element(struct List *elements, struct value_type *element);

int
new_object_member(struct List *members, struct member_type *member);

void
json_print_recursive(struct value_type *value);
#endif
